/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the license.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common;

// ToolBox
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.FileUtil;

// AddinCore
import org.enhydra.kelp.common.map.Mapper;
import org.enhydra.kelp.common.importer.ResourceFilter;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterFileNode;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterTemplateNode;

// JDK
import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;

//
public class PathUtil {

    // string not to be translated
    private final static String UNTITLED = "untitled";    // nores
    private final static String _PATH = "_PATH@";         // nores

    //
    public PathUtil() {}

    public static boolean isInputInSource(OtterProject project) {
        PathHandle inPath = null;
        PathHandle srcPath = null;
        String[]   srcs = new String[0];
        boolean    inSource = false;

        inPath = PathHandle.createPathHandle(project.getDeployInputPath());
        srcs = project.getSourcePathArray();
        for (int i = 0; i < srcs.length; i++) {
            srcPath = PathHandle.createPathHandle(srcs[i]);
            if (srcPath.parentOf(inPath) || srcPath.equals(inPath)) {
                inSource = true;
                break;
            }
        }
        return inSource;
    }

    public static boolean isTemplate(OtterFileNode node) {
        boolean    temp = false;
        PathHandle handleNode = null;
        PathHandle handleDeploy = null;

        handleNode = PathHandle.createPathHandle(node.getFilePath());
        if (handleNode.hasExtension(Constants.TYPE_IN)) {
            OtterProject project = node.getProject();

            handleDeploy =
                PathHandle.createPathHandle(project.getDeployInputPath());
            if (handleDeploy.parentOf(handleNode)) {
                temp = true;
            }
        }
        return temp;
    }

    public static String getDefaultDeployArchivePath(OtterProject project) {
        File         f = null;
        StringBuffer path = new StringBuffer();
        int          type = OtterProject.TYPE_UNKNOWN;

        f = new File(project.getDeployRootPath());
        type = project.getDeployType();
        path.append(f.getAbsolutePath());
        path.append(File.separator);
        path.append(Constants.DIR_LIB);
        path.append(File.separator);
        if (project == null) {
            path.append(PathUtil.UNTITLED);
        } else {
            path.append(project.getProjectRootName());
        }
        path.append('.');
        switch (type) {
        case OtterProject.TYPE_WEBAPP:
            path.append(Constants.TYPE_WAR);
            break;
        case OtterProject.TYPE_EN3APP:
        case OtterProject.TYPE_SERVICE:
        case OtterProject.TYPE_UNKNOWN:
            path.append(Constants.TYPE_JAR);
            break;
        }
        return PathHandle.createPathString(path.toString());
    }

    public static String[][] expandReplacementTable(OtterProject project,
            String[][] table) {
        for (int i = 0; i < table.length; i++) {
            if (table[i][0].endsWith(PathUtil._PATH)) {
                table[i][1] = PathUtil.expandPathRelativeToProject(project,
                        table[i][1]);
            }
        }
        return table;
    }

    public static String getDefaultDeployBootstrapPath(OtterProject project) {
        String       conf = PathUtil.getDeployConfPath(project);
        String       def = new String();
        StringBuffer path = new StringBuffer();
        PathHandle   ph = null;

        if (ToolBoxInfo.isEnhydra3()) {
            path.append(conf);
            path.append(File.separator);
            path.append(Constants.FILE_MULTISERVER_CONF);
        } else if (project.getDeployType() == OtterProject.TYPE_SERVICE) {
            path.append(PathUtil.getAutoDeployPath(OtterProject.TYPE_SERVICE));
        } else {
            path.append(conf);
            path.append(File.separator);
            path.append(Constants.FILE_BOOTSTRAP_CONF);
        }
        ph = PathHandle.createPathHandle(path.toString());
        if (!ph.isFile()) {
            ph.createPathHandle(Backward.createDefaultBootstrapPath(path.toString(),
                    project));
        }
        return ph.getPath();
    }

    public static String getDeployConfPath(OtterProject project) {
        return PathUtil.getDeployConfPath(project.getDeployRootPath());
    }

    public static String getDeployConfPath(String root) {
        StringBuffer path = new StringBuffer();

        path.append(root);
        path.append(File.separator);
        path.append(Constants.DIR_CONF);
        return PathHandle.createPathString(path.toString());
    }

    public static String getDeployContentPath(OtterProject project) {
        return PathUtil.getDeployContentPath(project,
                                             project.getDeployRootPath());
    }

    public static String getDeployContentPath(OtterProject project,
                                              String root) {
        StringBuffer buf = new StringBuffer();
        String       path = new String();
        Mapper       mapper = null;
        int          type = OtterProject.TYPE_UNKNOWN;

        if (project != null) {
            type = project.getDeployType();
        }
        switch (type) {
        case OtterProject.TYPE_SERVICE:
            path = project.getClassOutputPath();
            break;
        case OtterProject.TYPE_EN3APP:
            String resourcePath = new String();
            String sourcePath = new String();

            resourcePath = project.getDeployResourcePath();
            sourcePath = PathUtil.getSourcePathOf(project, resourcePath);
            mapper = new Mapper(project);
            path = mapper.getMappedOutputPath(sourcePath, resourcePath);
            break;
        default:
            buf.append(root);
            buf.append(File.separator);
            buf.append(Constants.DIR_CONTENT);
            path = buf.toString();
            break;
        }
        return PathHandle.createPathString(path);
    }

    public static String getDefaultDeployResourcePath(OtterProject project,
            String appPackPath, String sourcePath) {
        String         path = sourcePath;
        StringBuffer   searchRootPath = new StringBuffer();
        File           firstFound = null;
        ResourceFilter resourceFilter = null;

        resourceFilter = new ResourceFilter();
        resourceFilter.setResourcePath(true);
        resourceFilter.setProject(project);
        searchRootPath.append(sourcePath);
        searchRootPath.append(File.separator);
        searchRootPath.append(appPackPath);
        firstFound = FileUtil.findFirst((FileFilter) resourceFilter,
                                        searchRootPath.toString());
        if (firstFound == null) {
            resourceFilter.setResourcePath(false);
            firstFound = FileUtil.findFirst((FileFilter) resourceFilter,
                                            searchRootPath.toString());
        }
        if (firstFound != null) {
            path = firstFound.getParent();
        }
        return PathHandle.createPathString(path);
    }

    /**
     * Method declaration
     *
     *
     * @param node
     * @param propertyName
     * @param inFilename
     *
     * @see
     */
    public static void putFileRelativeToProject(OtterNode node,
                                                String propertyName,
                                                String inFilename) {
        String outFilename = compressPathRelativeToProject(node, inFilename);

        node.setProperty(propertyName, outFilename);
    }

    /**
     * Method declaration
     *
     *
     * @param node
     * @param propertyName
     *
     * @return
     *
     * @see
     */
    public static File getFileRelativeToProject(OtterNode node,
                                                String propertyName) {
        File   file = null;
        String filename = node.getProperty(propertyName);

        if ((filename != null) && (filename.trim().length() > 0)) {
            filename = PathUtil.expandPathRelativeToProject(node, filename);
            file = new File(filename);
        }
        return file;
    }

    /**
     * Method declaration
     *
     *
     * @param node
     * @param inPath
     *
     * @return
     *
     * @see
     */
    public static String compressPathRelativeToProject(OtterNode node,
            String in) {
        PathHandle ph = null;
        String     out = null;

        ph = PathHandle.createPathHandle(node.getProject().getRootPath());
        out = ph.getRelativePath(in);
        return out;
    }

    /**
     * Method declaration
     *
     *
     * @param node
     * @param inPath
     *
     * @return
     *
     * @see
     */
    public static String expandPathRelativeToProject(OtterNode node,
            String inPath) {
        PathHandle ph = null;

        ph = PathHandle.createPathHandle(node.getProject().getRootPath());
        ph = ph.expandRelativePath(inPath);
        return ph.getPath();
    }

    public static OtterTemplateNode[] getInputTemplates(OtterProject project) {
        OtterFileNode[]     input = null;
        OtterTemplateNode[] templates = null;
        OtterNodeFactory    factory = null;
        ArrayList           list = new ArrayList();

        input = project.getAllInput();
        factory = project.getNodeFactory();
        for (int i = 0; i < input.length; i++) {
            PathHandle cursor = null;

            cursor = PathHandle.createPathHandle(input[i].getFilePath());
            if (cursor.hasExtension(Constants.TYPE_IN)) {
                OtterTemplateNode newNode = null;

                newNode = factory.getTemplateNode(input[i]);
                list.add(newNode);
            }
        }
        list.trimToSize();
        templates = new OtterTemplateNode[list.size()];
        templates = (OtterTemplateNode[]) list.toArray(templates);
        list.clear();
        return templates;
    }

    public static OtterFileNode[] getInputPassthrough(OtterProject project) {
        OtterFileNode[] input = null;
        OtterFileNode[] pass = null;
        ArrayList       list = new ArrayList();

        input = project.getAllInput();
        for (int i = 0; i < input.length; i++) {
            PathHandle cursor = null;

            cursor = PathHandle.createPathHandle(input[i].getFilePath());
            if (cursor.hasExtension(Constants.TYPE_IN)) {

                // skip templates
            } else {
                list.add(input[i]);
            }
        }
        list.trimToSize();
        pass = new OtterFileNode[list.size()];
        pass = (OtterFileNode[]) list.toArray(pass);
        list.clear();
        return pass;
    }

    public static String getSourcePathOf(OtterProject project,
                                         String sourceFile) {
        String[]   path = project.getSourcePathArray();
        PathHandle sourcePath = null;
        PathHandle filePath = null;
        PathHandle parentPath = null;

        filePath = PathHandle.createPathHandle(sourceFile);
        sourcePath = filePath.getParent();
        for (int i = 0; i < path.length; i++) {
            parentPath = PathHandle.createPathHandle(path[i]);
            if (parentPath.parentOf(filePath)) {
                sourcePath.setPath(parentPath.getPath());
                break;
            }
        }
        return sourcePath.getPath();
    }

    public static String getOutputPath(OtterTemplateNode node) {
        PathHandle   phInput = null;
        PathHandle   phOut = null;
        StringBuffer path = new StringBuffer();
        String       thisPath = null;
        OtterProject project = null;

        project = node.getProject();
        phInput = PathHandle.createPathHandle(project.getDeployInputPath());
        thisPath = node.getFilePath();
        path.append(project.getDeployRootPath());
        if (phInput.parentOf(thisPath)) {
            int start = project.getDeployInputPath().length();
            int end = thisPath.length() - 3;

            path.append(thisPath.substring(start, end));
        } else {
            File thisFile = new File(thisPath);

            path.append(File.separator);
            path.append(thisFile.getName().substring(0,
                                                     thisFile.getName().length()
                                                     - 3));
        }
        phOut = PathHandle.createPathHandle(path.toString());
        return phOut.getPath();
    }

    public static String getAutoDeployPath(int type) {
        StringBuffer buf = new StringBuffer();

        buf.append(ToolBoxInfo.getEnhydraRoot());
        buf.append(File.separator);
        if (type == OtterProject.TYPE_EN3APP) {
            buf.append("apps");
        } else {
            buf.append("deploy");
        }
        return PathHandle.createPathString(buf.toString());
    }

}
