/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.kelp.common.importer;

// ToolBox
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.FilenameFilter;
import org.enhydra.tool.common.PathHandle;

// AddinCore
import org.enhydra.kelp.common.Constants;
import org.enhydra.kelp.common.PathUtil;
import org.enhydra.kelp.common.ValidationException;
import org.enhydra.kelp.common.ValidationUtil;
import org.enhydra.kelp.common.map.MapEntry;
import org.enhydra.kelp.common.map.MapUtil;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.PropertyKeys;

// JDK
import java.io.File;
import java.util.ResourceBundle;

//
public class ImportPaths implements PropertyKeys {

    // strings not to be translated
    static ResourceBundle  res =
        ResourceBundle.getBundle("org.enhydra.kelp.common.Res");    // nores
    private final String   PRESENTATION = "presentation";           // nores
    private final String[] PACKAGE_KEYS = {
        ".business", ".data", '.' + PRESENTATION
    };                                                              // nores

    //
    private OtterProject   project = null;
    private String         pack = new String();
    private String[][]     packMap = new String[0][2];
    private String         deployRootPath = null;
    private String         resourcePath = null;
    private String         sourcePath = null;
    private String         rootPath = null;
    private String         defaultSourcePath = null;
    private String         defaultPackage = null;
    private ResourceFilter resourceFilter = new ResourceFilter();
    private JavaFilter     javaFilter = new JavaFilter();
    private MakefileReader makefileReader = new MakefileReader();

    public ImportPaths() {}

    protected void initProject() {
        FilenameFilter filter = null;
        File           firstFound = null;

        getProject().setWorkingPath(getRootPath());    // 1st set
        getProject().setDeployRootPath(getDeployRootPath());
        getProject().setDeployResourcePath(getResourcePath());
        getProject().setClassOutputPath(getClassOutputPath());
        getProject().setPackageMap(getPackageMap());

        // determine type
        filter = new FilenameFilter();
        filter.setDirectoryValid(false);
        filter.setIncludeName(Constants.FILE_WEB_XML);
        firstFound = FileUtil.findFirst(filter, getSourcePath());
        if (firstFound == null) {
            filter.setIncludeName(Constants.FILE_ENHYDRA_SERVICE_XML);
            firstFound = FileUtil.findFirst(filter, getSourcePath());
            if (firstFound == null) {
                getProject().setDeployType(OtterProject.TYPE_EN3APP);
            } else {
                getProject().setDeployType(OtterProject.TYPE_SERVICE);
            }
        } else {
            getProject().setDeployType(OtterProject.TYPE_WEBAPP);
        }
        getProject().setProperty(PropertyKeys.NAME_LIBRARIES,
                                 PropertyKeys.VALUE_ENHYDRA);
    }

    public OtterProject getProject() {
        return project;
    }

    public MakefileReader getMakefileReader() {
        return makefileReader;
    }

    public JavaFilter getJavaFilter() {
        return javaFilter;
    }

    public void setProject(OtterProject p) {
        project = p;
        resourceFilter.setProject(project);
        makefileReader.setProject(project);
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getPackage() {
        return pack;
    }

    /**
     * Method declaration
     *
     *
     * @param pac
     *
     * @exception ValidationException
     */
    public void setPackage(String pac) throws ValidationException {
        if (ValidationUtil.isJavaPackage(pac)) {
            pack = pac;
        } else {
            throw new ValidationException(res.getString("Not_a_valid_package")
                                          + pac);
        }
    }

    public void initPackageMap(boolean write) {
        String[][] readMap = makefileReader.getPackageMap();

        makefileReader.invalidate();
        String[][] defMap = getDefaultPackageMap();
        MapEntry[] entries = MapUtil.combine(MapUtil.toEntryArray(readMap),
                                             MapUtil.toEntryArray(defMap));

        packMap = MapUtil.toStringArray(entries);
        if (write) {
            getProject().setPackageMap(packMap);
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String[][] getPackageMap() {
        return packMap;
    }

    /**
     * Method declaration
     *
     *
     * @param m
     */
    public void setPackageMap(String[][] m) {
        packMap = m;
    }

    /**
     *
     */
    protected String getPackagePath() {
        return createPackagePath(getPackage());
    }

    /**
     *
     */
    private String createPackagePath(String packPath) {
        String path = new String();

        if (packPath != null) {
            path = packPath.replace('.', '/');
        }
        return path;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getResourcePath() {
        if (resourcePath == null) {
            resourcePath = getDefaultResourcePath();
        }
        return resourcePath;
    }

    /**
     * Method declaration
     *
     *
     * @param dir
     *
     * @exception ValidationException
     */
    public void setResourcePath(String path) throws ValidationException {
        if (ValidationUtil.isDirectory(path)) {
            resourcePath = PathHandle.createPathString(path);
        } else {
            throw new ValidationException(res.getString("Resource_not_found")
                                          + path);
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getRootPath() {
        return rootPath;
    }

    /**
     * Method declaration
     *
     *
     * @param dir
     *
     * @exception ValidationException
     */
    public void setRootPath(String path) throws ValidationException {
        if (ValidationUtil.isDirectory(path)) {
            rootPath = PathHandle.createPathString(path);
            defaultSourcePath = null;
        } else {
            throw new ValidationException(res.getString("Project_root_not_found"));
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getSourcePath() {
        return sourcePath;
    }

    /**
     * Method declaration
     *
     *
     * @param dir
     *
     * @exception ValidationException
     */
    public void setSourcePath(String path) throws ValidationException {
        File dir = null;

        if ((path == null) || (path.trim().length() == 0)) {
            throw new ValidationException(res.getString("Source_path_empty"));
        } else {
            dir = new File(path);
        }
        if (ValidationUtil.isDirectory(dir)) {
            sourcePath = PathHandle.createPathString(dir);
            defaultPackage = null;
        } else {
            StringBuffer message = new StringBuffer();

            if (dir == null) {
                message.append(res.getString("Source_null"));
            } else {
                message.append(res.getString("Source_not_found"));
                message.append(dir.getAbsolutePath());
            }
            throw new ValidationException(message.toString());
        }
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getDefaultPackage() {
        File firstFound = null;
        int  index = 0;

        if (defaultPackage == null) {
            if (getSourcePath() != null) {
                firstFound = FileUtil.findFirst(javaFilter, getSourcePath());
            }
            if (firstFound != null) {
                defaultPackage = ImportTool.readPackage(firstFound);
                for (int i = 0; i < PACKAGE_KEYS.length; i++) {
                    index = defaultPackage.indexOf(PACKAGE_KEYS[i]);
                    if (index > 0) {
                        defaultPackage = defaultPackage.substring(0, index);
                        break;
                    }
                }
            }
        }
        return defaultPackage;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String[][] getDefaultPackageMap() {
        String[][]   defaultMap = new String[1][2];
        StringBuffer buf = new StringBuffer();
        File         firstFound = null;

        if (getPackage().length() == 0) {
            buf.append(getDefaultPackage());
        } else {
            buf.append(getPackage());
        }
        if (buf.toString().trim().length() > 0) {
            buf.append('.');
        }
        buf.append(PRESENTATION);
        defaultMap[0][0] = getResourcePath();
        defaultMap[0][1] = buf.toString();

        // That normally works, but lets check a different parallel
        // html to java structure.
        if (getResourcePath() != null) {
            File resource = new File(getResourcePath());

            firstFound = FileUtil.findFirst(resourceFilter, resource, 0);
        }
        if (firstFound != null) {
            File   parent = firstFound.getParentFile().getParentFile();
            File[] dirList = parent.listFiles();
            File[] javaList = null;
            String baseName = firstFound.getName();

            baseName = baseName.substring(0, baseName.indexOf('.'));
            for (int i = 0; i < dirList.length; i++) {
                if (dirList[i].isDirectory()) {
                    javaList = dirList[i].listFiles(javaFilter);
                    for (int j = 0; j < javaList.length; j++) {
                        if (javaList[j].getName().startsWith(baseName)) {
                            defaultMap[0][0] =
                                PathHandle.createPathString(firstFound.getParentFile());
                            defaultMap[0][1] =
                                ImportTool.readPackage(javaList[j]);
                            break;
                        }
                    }
                }
            }
        }
        return defaultMap;
    }

    public String getDefaultResourcePath() {
        String srcPath = new String();
        String appPackagePath = new String();
        String path = new String();

        if (getPackage() == null) {
            appPackagePath = createPackagePath(getDefaultPackage());
        } else {
            appPackagePath = getPackagePath();
        }
        if (getSourcePath() != null) {
            srcPath = getSourcePath();
        } else if (getDefaultSourcePath() != null) {
            srcPath = getDefaultSourcePath();
        } else if (getRootPath() != null) {
            srcPath = getRootPath();
        }
        path = PathUtil.getDefaultDeployResourcePath(project, appPackagePath,
                srcPath);
        return path;
    }

    /**
     * Method declaration
     *
     *
     * @return
     */
    public String getDefaultSourcePath() {
        File   firstFound = null;
        String inPack = new String();
        int    depth = 0;

        if (defaultSourcePath == null) {
            if (rootPath != null) {
                defaultSourcePath = rootPath;
                firstFound = FileUtil.findFirst(javaFilter, rootPath);
            }
            if (firstFound != null) {
                inPack = ImportTool.readPackage(firstFound);
                if (ValidationUtil.isJavaPackage(inPack)) {
                    if (inPack.length() > 0) {
                        depth = charCount(inPack, '.') + 1;
                    }
                }
                File foundParent = firstFound.getParentFile();

                while (depth > 0) {
                    foundParent = foundParent.getParentFile();
                    depth--;
                }
                defaultSourcePath = foundParent.getAbsolutePath();
            }
        }
        defaultSourcePath = PathHandle.createPathString(defaultSourcePath);
        return defaultSourcePath;
    }

    public void setDeployRootPath(String p) {
        deployRootPath = p;
    }

    public String getDeployRootPath() {
        String p = deployRootPath;

        if (p == null) {
            StringBuffer buf = new StringBuffer();

            buf.append(getRootPath());
            buf.append(File.separator);
            buf.append(Constants.DIR_OUTPUT);
            p = PathHandle.createPathString(buf.toString());
        }
        return p;
    }

    // /
    // / PROTECTED
    // /
    protected String getClassOutputPath() {
        StringBuffer buf = new StringBuffer();

        buf.append(getRootPath());
        buf.append(File.separator);
        buf.append(Constants.DIR_CLASSES);
        return PathHandle.createPathString(buf.toString());
    }

    /**
     * Method declaration
     *
     *
     * @param in
     * @param lookFor
     *
     * @return
     */
    private int charCount(String in, char lookFor) {
        String search = new String(in);
        int    index = search.indexOf(lookFor);
        int    count = 0;

        while (index > 0) {
            count++;
            search = search.substring(index + 1);
            index = search.indexOf(lookFor);
        }
        return count;
    }

}
