/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.codegen;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.codegen.wizard.CodeGenWizard;
import org.enhydra.tool.codegen.wizard.CodeGenPage;
import org.enhydra.tool.codegen.internal.Enhydra3AppGenerator;
import org.enhydra.tool.codegen.internal.ServiceGenerator;
import org.enhydra.tool.codegen.internal.WebAppGenerator;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.ResUtil;

// Standard imports
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Properties;
import java.util.ArrayList;
import java.beans.Beans;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.Frame;
import javax.swing.JOptionPane;
import java.util.ResourceBundle;

/**
 * Main class for CodeGen wizard and command line interface.
 */
public class CodeGen implements Constants {

    /**
     * When the debug flag is true, stack traces are printed
     * for all exceptions.
     */
    static ResourceBundle       res =
        ResourceBundle.getBundle("org.enhydra.tool.codegen.Res");
    public static boolean       debug = false;

    /**
     * When the verbose flag is true, file names stream out
     * during file operations.
     */
    public static boolean       verbose = false;

    //
    private static final String GENERATOR = "external.generator";    // nores
    private static final String CLASS_WEB_GEN =
        "org.enhydra.tool.codegen.internal.WebAppGenerator";         // nores
    private static final String CLASS_EN3_GEN =
        "org.enhydra.tool.codegen.internal.Enhydra3AppGenerator";    // nores
    private static final String CLASS_SERVICE_GEN =
        "org.enhydra.tool.codegen.internal.ServiceGenerator";        // nores

    //
    private Generator[]         generators = new Generator[0];
    private Generator           selection = null;
    private Properties          properties = new Properties();
    private int                 option = JOptionPane.CANCEL_OPTION;
    private boolean             swing = false;

    //
    // PUBLIC CONSTRUCTORS
    //

    /**
     * Create an Code Generator in command line mode so that no
     * swing components are instantiated.
     *
     * @throws GeneratorException
     * Thrown if no generators could be instantiated. Warnings
     * print out for each generators that could not be instantiated.
     */
    public CodeGen() throws GeneratorException {
        this(false);
    }

    /**
     * Create an Code Generator and read in the
     * codegen.properties file. Use Beans instantiate to
     * create instances of each generator listed in
     * codegen.properties. Also pass the properties object
     * into each generator.
     *
     * @throws GeneratorException
     * Thrown if no generators could be instantiated. Warnings
     * print out for each generators that could not be instantiated.
     *
     * @param swing
     * If true, swing components will be used for user interaction.
     */
    public CodeGen(boolean useSwing) throws GeneratorException {
        setSwing(useSwing);
        readProperties();
        initGenerators();
        if (generators.length == 0) {
            throw new GeneratorException(res.getString("No_generators"));
        }
    }

    /**
     * Create a Code Generator using a specified implementation
     * of the Generator class. This constructor is used by Kelp
     * to create seperate wizards for each generator.
     *
     * @throws GeneratorException
     * Thrown if properties fail to initialize.
     *
     * @param selection
     * The generator implementation to use for all
     * generation operations.
     */
    public CodeGen(Generator sel) throws GeneratorException {
        selection = sel;
        setSwing(true);
        readProperties();
        generators = new Generator[1];
        generators[0] = selection;
        selection.setProperties(properties);
    }

    //
    // PUBLIC METHODS
    //
    public boolean isSwing() {
        return swing;
    }

    public void setSwing(boolean b) {
        swing = b;
    }

    /**
     * Get an integer representing the button selected
     * by the use to close the wizard dialog.
     *
     * @return
     * CodeGenDialog.CHOICE_FINISH or CodeGenDialog.CHOICE_CANCEL.
     * The CHOICE_FINISH value indicates that code generation was
     * attempted.
     */
    public int getOption() {
        return option;
    }

    /**
     * Get generator implementation to use for all generation
     * operations.
     *
     * @return
     * A specific implementation of the generator interface.
     */
    public Generator getSelection() {
        return selection;
    }

    public void setSelection(Generator sel) {
      selection = sel;
    }

    /**
     * Open the wizard dialog where a user can select what generator
     * to use, specify generation options and generate the project.
     *
     * @param host
     * A component that can act as the owner of the modal dialog. May be
     * null, Frame or Dialog.
     *
     * @return
     * Returns an array of generated file references.
     */
    public File[] invokeWizard(Component host) {
        CodeGenWizard wizard = null;
        File[]               files = new File[0];

        wizard = new CodeGenWizard(this);
        if (selection != null) {
            try {
                wizard.setSelection(selection);
            } catch (GeneratorException e) {
                JOptionPane.showMessageDialog(host, e.getMessage(),
                                              res.getString("Unable_to_set"),
                                              JOptionPane.ERROR_MESSAGE);
                if (CodeGen.debug) {
                    e.printStackTrace();
                }
            }
        }
        option = wizard.showDialog(host);
        files = wizard.getGeneratedFiles();
        if (option == JOptionPane.CANCEL_OPTION) {

            // done
        } else {
            if (files == null || files.length == 0) {
                JOptionPane.showMessageDialog(host,
                                              res.getString("No_files_generated_"),
                                              res.getString("AppWizard_Complete"),
                                              JOptionPane.WARNING_MESSAGE);
            } else {
                String parentPath = files[0].getParent();

                for (int i = 0; i < files.length; i++) {
                    if (files[i].getParent().length() < parentPath.length()) {
                        parentPath = files[i].getParent();
                    }
                }
                String mess =
                    ResUtil.format(res.getString("Generated_0_files_in"),
                                   (new String()) + files.length, parentPath);

                JOptionPane.showMessageDialog(host, mess,
                                              res.getString("AppWizard_Complete"),
                                              JOptionPane.INFORMATION_MESSAGE);
            }
        }
        return files;
    }

    // /
    // / PRIVATE STATIC
    // /

    /**
     * Method declaration
     *
     * @exception GeneratorException
     */
    private void createProperties() throws GeneratorException {

        // create default property set
        Properties prop = new Properties();
        File       file = null;

        try {
            file = ToolBoxInfo.storeProperties(prop);
            System.out.println(ResUtil.format(res.getString("Configuration"),
                                              file));
        } catch (Exception e) {
            throw new GeneratorException(e,
                                         ResUtil.format(res.getString("Unable_to_write_file"),
                                                        ToolBoxInfo.getPropertyFilename()));
        }
    }

    // /
    // / PRIVATE
    // /
    public void setGenerators(Generator[] g) {
        generators = g;
    }

    public Generator[] getGenerators() {
        return generators;
    }

    /**
     * Method declaration
     *
     */
    public void initGenerators() throws GeneratorException {
        String    className;
        Object    newObject;
        ArrayList gList = new ArrayList();
        int       validCount = 0;

        // read exteneral generators
        for (int i = 0; i < 100; i++) {
            className = properties.getProperty(GENERATOR + '.' + i);
            newObject = null;
            if (className != null && className.trim().length() > 0) {
                try {
                    newObject = Beans.instantiate(getClass().getClassLoader(),
                                                  className);
                } catch (Exception e) {
                    newObject = null;
                    System.out.println(ResUtil.format(res.getString("AppWizard_Warning"),
                                                      className));
                    if (CodeGen.debug) {
                        System.out.println(TAB4 + e.toString());
                    }
                }
                if (newObject != null) {
                    if (newObject instanceof Generator) {
                        gList.add((Generator) newObject);
                    } else {
                        System.out.println(ResUtil.format(res.getString("AppWizard_Warning_Not"),
                                                          className));
                    }
                }
            }
        }

        // add internal generators
        gList.add(new WebAppGenerator());
        if (!ToolBoxInfo.isEnhydra3()) {
            gList.add(new ServiceGenerator());
        }
        gList.add(new Enhydra3AppGenerator());
        for (int i = 0; i < gList.size(); i++) {
            Generator current = null;

            current = (Generator) gList.get(i);
            try {
                current.setProperties(properties);
                current.setSwing(isSwing());
                validCount++;
            } catch (GeneratorException e) {
                System.out.println(ResUtil.format(res.getString("Invalid_generator_0_"),
                                                  current.getCommandName()));
                System.out.println(e.getMessage());
                gList.set(i, new Object());
            }
        }
        if (validCount == 0) {
            throw new GeneratorException(res.getString("No_generators1"));
        } else {
            generators = new Generator[validCount];
        }
        validCount = 0;
        for (int i = 0; i < gList.size(); i++) {
            if (gList.get(i) instanceof Generator) {
                generators[validCount] = (Generator) gList.get(i);
                validCount++;
            }
        }
        gList.clear();
    }

    /**
     * Method declaration
     *
     */
    protected void list() {
        int count = generators.length;

        System.out.println(new String());
        if (count < 1) {
            System.out.println(res.getString("No_generators_to_list"));
        } else {
            System.out.println(res.getString("Available_generators_"));
            for (int i = 0; i < count; i++) {
                System.out.println(TAB4 + generators[i].getCommandName());
            }
            System.out.println(new String());
            System.out.println(res.getString("Generator"));
            for (int i = 0; i < count; i++) {
                StringBuffer buf = new StringBuffer();

                buf.append(generators[i].getCommandName());
                buf.append(TAB4);
                buf.append(generators[i].getDisplayName());
                buf.append('\n');
                buf.append(generators[i].getDescription());
                buf.append('\n');
                System.out.println(buf.toString());
            }
        }
        System.out.println(new String());
    }

    /**
     * Method declaration
     *
     *
     * @param args
     */
    protected void invokeGenerator(String[] args) throws GeneratorException {
        File[] files = new File[0];

        initSelection(args[0]);
        if (selection == null) {}

        // do nothing
        else {
            processGeneratorOptions(args);
            System.out.println(ResUtil.format(res.getString("Generator_0_"),
                                              selection.getCommandName()));
            GeneratorOption[] options = selection.getOptionSet().toArray();

            for (int i = 0; i < options.length; i++) {
                if (options[i].isRequired() || CodeGen.verbose) {
                    System.out.println(options[i].getDisplayName() + ':'
                                       + ' ' + options[i].getValue());
                }
            }
            files = selection.generate();
            System.out.println(ResUtil.format(res.getString("File_count_0_"),
                                              files.length));
        }
    }

    /**
     * Method declaration
     *
     * @param args
     * @exception GeneratorException
     */
    private void processGeneratorOptions(String[] args)
            throws GeneratorException {

        // Do basic validation and update option set
        // with argument values.
        GeneratorOption[] options = selection.getOptionSet().toArray();

        // Clear out values for required items.
        for (int i = 0; i < options.length; i++) {
            if (options[i].isRequired()) {
                options[i].clearValue();
            }
        }

        // Report any arguments we don't recognize and will ignore.
        for (int i = 1; i < args.length; i++) {
            if ((args[i].length() > 0) && (args[i].charAt(0) == '-')) {
                boolean found = false;

                // it is a name.
                for (int j = 0; j < options.length; j++) {
                    String paramName = '-' + options[j].getName();

                    if (paramName.equalsIgnoreCase(args[i])) {
                        found = true;
                        if (options[j].isBoolean()) {
                            options[j].setValue((new Boolean(true)).toString());
                        } else if (i + 1 < args.length) {
                            options[j].setValue(args[i + 1]);
                        }
                    }
                }
                if (!found) {
                    System.out.println(ResUtil.format(res.getString("Ignoring_unrecognized"),
                                                      args[i]));
                }
            }
        }
        StringBuffer buf = new StringBuffer();
        boolean      missing = false;

        for (int j = 0; j < options.length; j++) {
            if (options[j].isRequired()) {
                if (options[j].isEmpty()) {
                    if (!missing) {
                        missing = true;
                        buf.append(res.getString("Required_value"));
                        buf.append('\n');
                    }
                    buf.append(TAB4);
                    buf.append('-');
                    buf.append(options[j].getName());
                    buf.append('\n');
                }
            }
        }
        if (buf.length() > 0) {
            buf.append('\n');
            buf.append(res.getString("For_more_information"));
            buf.append(' ');
            buf.append(selection.getCommandName());
            throw new GeneratorException(buf.toString());
        }

        // do something wonderful
    }

    /**
     * Method declaration
     *
     * @param commandName
     */
    private void initSelection(String commandName) throws GeneratorException {
        selection = null;
        for (int i = 0; i < generators.length; i++) {
            if (generators[i].getCommandName().equalsIgnoreCase(commandName)) {
                selection = generators[i];
                break;
            }
        }
        if (selection == null) {
            System.out.println(ResUtil.format(res.getString("_0_is_not_a_valid"),
                                              commandName));
        }
    }

    /**
     * Method declaration
     *
     *
     * @exception GeneratorException
     */
    private void readProperties() throws GeneratorException {
        String fn = ToolBoxInfo.getPropertyFilename();
        File   file = new File(fn);

        if (!file.exists()) {
            createProperties();
        }
        try {
            properties = ToolBoxInfo.loadProperties();
        } catch (ToolException e) {
            throw new GeneratorException(e, res.getString("Unable_to_read"));
        }
    }

    /**
     * Method declaration
     *
     * @param commandName
     */
    protected void showGeneratorOptionHelp(String commandName) {
        try {
            initSelection(commandName);
        } catch (GeneratorException e) {
            e.printStackTrace();
        }
        if (selection == null) {
            System.out.println(ResUtil.format(res.getString("Unable_to_show"),
                                              commandName));
        } else {
            GeneratorOption[] options = selection.getOptionSet().toArray();

            if (options.length == 0) {
                System.out.println(ResUtil.format(res.getString("No_options_available"),
                                                  commandName));
            } else {
                for (int i = 0; i < options.length; i++) {
                    StringBuffer buf = new StringBuffer();

                    buf.append(' ');
                    buf.append('-');
                    buf.append(options[i].getName());
                    if (options[i].isBoolean()) {

                        // skip placeholder
                    } else {
                        buf.append(' ');
                        buf.append(options[i].getDisplayName());
                    }
                    buf.append(TAB4);
                    buf.append(options[i].getDescription());
                    System.out.println(buf.toString());
                }
            }
        }
    }

}
