/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.codegen.internal;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.codegen.ProjectGenerator;
import org.enhydra.tool.codegen.GeneratorException;
import org.enhydra.tool.codegen.wizard.AppOptionPanel1;
import org.enhydra.tool.codegen.wizard.ProjectOptionPanel2;
import org.enhydra.tool.codegen.wizard.ProjectOptionPanel3;
import org.enhydra.tool.codegen.wizard.CodeGenPanel;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.Replacement;
import org.enhydra.tool.common.ReplacementSet;
import org.enhydra.tool.common.ResUtil;
import org.enhydra.tool.common.Template;
import org.enhydra.tool.common.TemplateFilter;
import org.enhydra.tool.common.TemplateTool;
import org.enhydra.tool.common.ToolException;
import org.enhydra.tool.common.event.TemplateEvent;
import org.enhydra.tool.common.event.TemplateListener;
import org.enhydra.tool.configure.ConfigTool;

// Standard imports
import java.io.BufferedReader;
import java.io.IOException;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.StringTokenizer;
import java.util.Vector;

//
abstract public class AppGenerator extends ProjectGenerator {
    private String         client = null;
    private String[][]     addinMap = null;
    private CodeGenPanel[] panels = null;

    public AppGenerator() {
        setReplacementSet(new AppReplacementSet());
        setOptionSet(new AppOptionSet());
    }

    // override ProjectGenerator
    protected String[] getInputIncludes() {
        ArrayList list = null;
        String[]  includes = new String[0];

        list = new ArrayList(Arrays.asList(super.getInputIncludes()));
        list.add(getClient());
        list.trimToSize();
        includes = new String[list.size()];
        includes = (String[]) list.toArray(includes);
        list.clear();
        return includes;
    }

    // override ProjectGenerator
    public void initReplacementSet() throws GeneratorException {
        super.initReplacementSet();
        try {
            String       content = null;
            String       app = null;
            StringBuffer buf = new StringBuffer();

            // client
            if (getClient().equalsIgnoreCase(TYPE_WML)) {
                content = CONTENT_WML;
            } else {

                // chtml, html, xhtml, etc.
                content = CONTENT_HTML;
            }
            app =
                getReplacementSet().lookup(AppReplacementSet.at_PROJECT_at).getReplaceWith()[0];
            buf.append(Character.toUpperCase(app.charAt(0)));
            if (app.length() > 1) {
                buf.append(app.substring(1));
            }
            app = buf.toString();
            getReplacementSet().lookup(AppReplacementSet.APP).setReplaceWith(app);
            getReplacementSet().lookup(AppReplacementSet.CLIENT).setReplaceWith(getClient().toUpperCase());
            getReplacementSet().lookup(AppReplacementSet.CONTENT_TYPE).setReplaceWith(content);
            getReplacementSet().lookup(AppReplacementSet.APP).setReplaceWith(app);
        } catch (ToolException e) {
            e.printStackTrace();
            throw new GeneratorException(e,
                                         res.getString("Unable_to_init_rep"));
        }
    }

    /**
     * Get the default option panels and preset the option set
     * for each panel.
     *
     * @return
     * An array containing the three default option panels.
     */
    public CodeGenPanel[] getWizardPanels() throws GeneratorException {

        // don't initialize until called.
        if (panels == null && isSwing()) {
            panels = new CodeGenPanel[3];
            panels[0] = new AppOptionPanel1();
            panels[1] = new ProjectOptionPanel2();
            panels[2] = new ProjectOptionPanel3();
            for (int i = 0; i < panels.length; i++) {
                panels[i].setOptionSet(getOptionSet());
                panels[i].readOptionSet();
            }
        }
        return panels;
    }

    protected String getClient() {
        if (client == null) {
            try {
                client =
                    getOptionSet().lookup(AppOptionSet.CLIENT).getValue().toUpperCase();
            } catch (GeneratorException e) {
                e.printStackTrace();
                client = null;
                return TYPE_HTML;
            }
        }
        return client;
    }

    /**
     * Get the default table for mapping directories to
     * packages that maps resources directory to the
     * presentaion package and the resource media directory
     * to the a media package.
     *
     * @param appPackage
     * This is the application package to prefix to each mapped
     * package name.
     *
     * @return
     * The default package map for template driven generators.
     */
    public String[][] expandAddinPackageMap() throws GeneratorException {
        String[][]   exMap = new String[0][0];
        TemplateTool tool = null;

        tool = new TemplateTool();
        tool.setReplacements(getReplacementSet().toArray());
        if (addinMap == null) {
            initAddinPackageMap();
        }
        exMap = new String[addinMap.length][addinMap[0].length];
        for (int i = 0; i < exMap.length; i++) {
            for (int j = 0; j < exMap[i].length; j++) {
                exMap[i][j] = addinMap[i][j];
            }
        }
        for (int i = 0; i < exMap.length; i++) {
            for (int j = 0; j < exMap[i].length; j++) {
                exMap[i][j] = tool.lineSearchAndReplace(exMap[i][j]);
            }
        }
        return exMap;
    }

    public String[][] getAddinPackageMap() throws GeneratorException {
        if (addinMap == null) {
            initAddinPackageMap();
        }
        return addinMap;
    }

    public void setAddinPackageMap(String[][] m) {
        addinMap = m;
    }

    private void initAddinPackageMap() throws GeneratorException {
        addinMap = new String[2][2];
        StringBuffer buf = new StringBuffer();

        buf.append(getDestination());
        buf.append(File.separator);
        buf.append(DIR_SRC);
        buf.append(File.separator);
        buf.append(DIR_AT_PACKAGE_DIR_AT);
        buf.append(File.separator);
        buf.append(DIR_RESOURCES);
        addinMap[0][0] = buf.toString();
        buf.setLength(0);
        buf.append("@JAVA_PACKAGE@");    // nores
        buf.append('.');
        buf.append(DIR_PRESENTATION);
        addinMap[0][1] = buf.toString();
        buf.setLength(0);
        buf.append(getDestination());
        buf.append(File.separator);
        buf.append(DIR_SRC);
        buf.append(File.separator);
        buf.append(DIR_AT_PACKAGE_DIR_AT);
        buf.append(File.separator);
        buf.append(DIR_RESOURCES);
        buf.append(File.separator);
        buf.append(DIR_MEDIA);
        addinMap[1][0] = buf.toString();
        addinMap[1][1] = DIR_MEDIA;
    }

    protected String[] getDefaultAddinSteps() {
        String[] steps = new String[4];

        steps[0] = res.getString("StepsKelp0");
        steps[1] = res.getString("StepsKelp1");
        steps[2] = res.getString("StepsKelp2");
        steps[3] = res.getString("StepsKelp3");
        return steps;
    }

    protected String[] getDefaultShellSteps() {
        String[]     steps = new String[3];
        StringBuffer buf = new StringBuffer();

        buf.append(res.getString("StepsShell0a"));
        buf.append(res.getString("StepsShell0b"));
        buf.append(res.getString("StepsShell0c"));
        buf.append(res.getString("StepsShell0d"));
        buf.append(res.getString("StepsShell0e"));
        buf.append(res.getString("StepsShell0f"));
        buf.append(res.getString("StepsShell0g"));
        buf.append(res.getString("StepsShell0h"));
        steps[0] = buf.toString();
        steps[1] = res.getString("StepsShell1");
        buf.setLength(0);
        buf.append(res.getString("StepsShell2a"));
        buf.append(res.getString("StepsShell2b"));
        steps[2] = buf.toString();
        return steps;
    }

}
