/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 * Paul Mahar
 *
 */
package org.enhydra.tool.common;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.AboutBox;
import org.enhydra.tool.common.ButtonPanel;
import org.enhydra.tool.common.InnerPanel;
import org.enhydra.tool.common.ProgressMeter;
import org.enhydra.tool.common.SwingUtil;
import org.enhydra.tool.common.event.HelpEvent;
import org.enhydra.tool.common.event.HelpListener;

// Standard imports
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.ResourceBundle;
import java.awt.Component;
import java.awt.Container;
import java.awt.Dialog;
import java.awt.Frame;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.Point;
import java.awt.Window;
import javax.swing.JComponent;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JDialog;

//
abstract public class DialogHandler {

    //
    public static ResourceBundle res =
        ResourceBundle.getBundle("org.enhydra.tool.common.Res");    // nores

    //
    private boolean              standalone = false;
    private int option = JOptionPane.CANCEL_OPTION;
    private ProgressMeter        progress = null;
    private HelpListener[]       helpListeners = new HelpListener[0];

    public DialogHandler() {
        pmInit();
    }

    abstract public InnerPanel getInnerPanel();
    abstract public ButtonPanel getButtonPanel();
    abstract public ActionListener createButtonListener();
    abstract public String getTitle();

    public int getOption() {
      return option;
    }

    public void setOption(int op) {
      option = op;
    }

    //
    public synchronized void addHelpListener(HelpListener l) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(helpListeners));
        if (!list.contains(l)) {
            list.add(l);
            list.trimToSize();
            helpListeners = new HelpListener[list.size()];
            helpListeners = (HelpListener[]) list.toArray(helpListeners);
        }
        list.clear();
    }

    public synchronized void removeHelpListener(HelpListener l) {
        ArrayList list = null;

        list = new ArrayList(Arrays.asList(helpListeners));
        if (list.contains(l)) {
            list.remove(l);
            list.trimToSize();
            helpListeners = new HelpListener[list.size()];
            helpListeners = (HelpListener[]) list.toArray(helpListeners);
        }
        list.clear();
    }

    public void notifyHelpListeners(Object source) {
        HelpEvent event = null;

        if (source instanceof JComponent) {
            JComponent comp = (JComponent) source;

            event = new HelpEvent(comp.getTopLevelAncestor());
        } else {
            event = new HelpEvent(source);
        }
        for (int i = 0; i < helpListeners.length; i++) {
            helpListeners[i].onHelp(event);
        }
    }



    public void showAbout() {
        StringBuffer textBuf = new StringBuffer();
        StringBuffer titleBuf = new StringBuffer();
        AboutBox     about = null;
        Container    ancestor = null;

        ancestor = getInnerPanel().getTopLevelAncestor();
        textBuf.append("Enhydra Application Server 4.0");
//        textBuf.append(' ');
//        textBuf.append(ToolBoxInfo.getEnhydraVersion());
        textBuf.append('\n');
        textBuf.append(Constants.XMLC);
        textBuf.append(' ');
        textBuf.append(ToolBoxInfo.getXMLCVersion());
        if (ancestor instanceof Frame) {
            about = new AboutBox((Frame) ancestor);
        } else if (ancestor instanceof Dialog) {
            about = new AboutBox((Dialog) ancestor);
        } else {
            about = new AboutBox(new Frame());
        }
        titleBuf.append(Constants.KELP);
        titleBuf.append(' ');
        titleBuf.append(ToolBoxInfo.getToolBoxVersion());
        about.setTitleText(titleBuf.toString());
        about.setMediumText(textBuf.toString());
//        about.setSmallText(ToolBoxInfo.getCopyright());
        about.show();
        about.removeAll();
        about.dispose();
    }

    public int showDialog(Component parent) {
        JPanel  panelBottom = null;
        JDialog dialog = null;
        Point   cPoint;

        getInnerPanel().initPreferredSize();
        if (parent == null) {
            dialog = new JDialog();
        } else if (parent instanceof Frame) {
            dialog = new JDialog((Frame) parent);
        } else if (parent instanceof Dialog) {
            dialog = new JDialog((Dialog) parent);
        } else {
            dialog = new JDialog();
        }
        getButtonPanel().addActionListener(createButtonListener());
        if (helpListeners.length == 0) {
            getButtonPanel().removeHelp();
        }
        panelBottom = new JPanel();
        panelBottom.setLayout(new GridBagLayout());
        dialog.setTitle(getTitle());
        dialog.setModal(true);
        dialog.getContentPane().setLayout(new GridBagLayout());
        dialog.getContentPane().add(getInnerPanel(),
                                    new GridBagConstraints(0, 0, 1, 1, 0.75,
                                    0.75, GridBagConstraints.CENTER,
                                    GridBagConstraints.BOTH,
                                    new Insets(0, 0, 0, 0), 0, 0));
        dialog.getContentPane().add(getButtonPanel(),
                                    new GridBagConstraints(0, 1, 1, 1, 0.25,
                                    0.25, GridBagConstraints.CENTER,
                                    GridBagConstraints.BOTH,
                                    new Insets(0, 0, 0, 0), 0, 0));
        dialog.invalidate();
        dialog.doLayout();
        dialog.pack();
        cPoint = SwingUtil.getCenteringPoint(dialog.getSize());
        dialog.setLocation(cPoint);
        getInnerPanel().preShow();
        dialog.show();
        return getOption();
    }

    /**
     * Close the dialog and exit the VM if this wizard
     * was run as a standalone application.
     */
    public void closeWindow() {
        Window window = null;

        try {
            getInnerPanel().save();
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
        if (getInnerPanel().getTopLevelAncestor() instanceof Window) {
            window = (Window) getInnerPanel().getTopLevelAncestor();
        }
        if (window != null) {
            window.removeAll();
            window.dispose();
            clearAll();
        }
        if (standalone) {
            System.exit(0);
        }
    }

    /**
     * Remove all object references.
     */
    protected void clearAll() {
        getInnerPanel().clearAll();
        getButtonPanel().clearAll();
        helpListeners = new HelpListener[0];
    }

     public ProgressMeter getProgressMeter() {
        return progress;
    }

    public void openProgress() {
        progress.setModal(true);
        progress.setTitle(getProgressTitle());
        progress.setOwner((Window) getInnerPanel().getTopLevelAncestor());
        progress.startDialogThread();
    }

    //
    // ABSTRACT PROTECTED
    //
    abstract protected String getProgressTitle();

    //
    // PRIVATE
    //
    private void pmInit() {
        progress = new ProgressMeter();
        progress.setDisposeOnClose(false);
        progress.setTitle(getProgressTitle());
    }

}
