/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 */
package org.enhydra.tool.configure;

// ToolBox imports
import org.enhydra.tool.ToolBoxInfo;
import org.enhydra.tool.common.FileUtil;
import org.enhydra.tool.common.PathHandle;
import org.enhydra.tool.common.Replacement;
import org.enhydra.tool.common.ReplacementSet;
import org.enhydra.tool.common.TemplateTool;
import org.enhydra.tool.common.ToolException;

// Standard imports
import java.io.BufferedReader;
import java.io.IOException;
import java.io.File;
import java.io.FileFilter;
import java.util.Vector;
import java.util.ResourceBundle;

//
public class ConfigTool extends TemplateTool implements Constants {
    private static ResourceBundle localRes =
        ResourceBundle.getBundle("org.enhydra.tool.configure.Res");    // nores

    //
    private boolean               transformPath = true;
    private String                javaPath = ToolBoxInfo.getJavaPath();
    private String                enhydraRoot = ToolBoxInfo.getEnhydraRoot();
    private String                appPath = null;
    private FileFilter            fillFilter = null;

    //
    private final String          DIR_DIST = "dist";                  // nores
    private final String          DIR_LIB = "lib";                    // nores
    private final String          FILE_ENHYDRA_JAR = "enhydra.jar";    // nores

    //
    private final String          AT_ENHYDRA_DIR_AT = "@ENHYDRA_DIR@";    // nores
    private final String          AT_JDKDIR_AT = "@JDKDIR@";          // nores
    private final String          AT_OUTPUT_AT = "@OUTPUT@";          // nores
    private final String          AT_ENHYDRA_CLASSES_AT =
        "@ENHYDRA_CLASSES@";                                          // nores

    //
    public static final String    PROJECT_PATH = "@PROJECT_PATH@";          // nores
    public static final String    JAVA_PATH = "@JAVA_PATH@";          // nores
    public static final String    DEPLOY_PATH = "@DEPLOY_PATH@";      // nores
    public static final String    ENHYDRA_PATH = "@ENHYDRA_PATH@";    // nores
    public static final String[]  PATHS = {
        PROJECT_PATH, JAVA_PATH, DEPLOY_PATH, ENHYDRA_PATH
    };

    public static Replacement[] createReplacements(
            String appRoot,
            String deployPath,
            String javaHome) throws ConfigException {
        Replacement[] defRep = new Replacement[4];

        try {
            defRep[0] = new Replacement(ConfigTool.PROJECT_PATH,
              PathHandle.createPathString(appRoot));
            defRep[1] = new Replacement(ConfigTool.DEPLOY_PATH,
              PathHandle.createPathString(deployPath));
            defRep[2] = new Replacement(ConfigTool.JAVA_PATH,
              PathHandle.createPathString(javaHome));
            defRep[3] = new Replacement(ConfigTool.ENHYDRA_PATH,
                                        ToolBoxInfo.getEnhydraRoot());
        } catch (ToolException e) {
            throw new ConfigException(e,
                                      localRes.getString("Unable_to_create"));
        }
        return defRep;
    }

    public static String[] getSuffixArray() {
        String[] sufs = new String[PATHS.length];

        for (int i = 0; i < PATHS.length; i++) {
            sufs[i] = PATHS[i].substring(1);
        }
        return sufs;
    }

    public static Replacement[] createReplacements(String deployPath)
            throws ConfigException {
        File f = new File(deployPath);
        return ConfigTool.createReplacements(f.getParent(),
                                             deployPath,
                                             ToolBoxInfo.getJavaPath());
    }

    public static String[][] createReplacementStringArray(String appRoot,
                                                          String deployPath,
                                                          String javaP)
            throws ConfigException {
        Replacement[]  reps = ConfigTool.createReplacements(appRoot, deployPath, javaP);
        ReplacementSet set = new ReplacementSet(reps);

        return set.toStringArray();
    }

    public ConfigTool() {}

    public File[] createOutput() throws ToolException {
        if (isTransformPath()) {
            setReplacements(getTransformedReplacements());
        }
        return super.createOutput();
    }

    public boolean isTransformPath() {
        return transformPath;
    }

    public void setTransformPath(boolean b) {
        transformPath = b;
    }

    public String getAppPath() {
        return appPath;
    }

    public void setAppPath(String p) {
        appPath = p;
    }

    public String getEnhydraRoot() {
        if (enhydraRoot == null) {
            enhydraRoot = ToolBoxInfo.getEnhydraRoot();
        }
        return enhydraRoot;
    }

    public void setEnhydraRoot(String p) {
        enhydraRoot = p;
    }

    public String getJavaPath() {
        if (javaPath == null) {
            javaPath = ToolBoxInfo.getJavaPath();
        }
        return javaPath;
    }

    public void setJavaPath(String p) {
        javaPath = p;
    }

    public void configure() throws ConfigException {
        File   dest = null;
        File   source = null;
        File[] output = new File[0];
        String[] types = new String[1];

        types[0] = TYPE_IN;

        try {
            if (getAppPath() == null) {
                source = new File(getEnhydraRoot() + File.separator
                                  + DIR_DIST);
                dest = new File(getEnhydraRoot());
                initEnhydraReplacements();
            } else {
                source = new File(getAppPath() + File.separator + DIR_INPUT);
                dest = new File(getAppPath() + File.separator + DIR_OUTPUT);
                initAppReplacements(dest);
            }
            setOverwrite(!isSwing());
            initTemplates(source, dest, types);
            output = createOutput();
        } catch (ToolException e) {
            throw (new ConfigException(e,
                                       localRes.getString("Unable_to_complete")));
        }
        if (output.length == 0 && isEcho()) {
            System.out.println(localRes.getString("No_templates"));
        }
    }

    public void initReplacements(String[][] strings) throws ConfigException {
        Replacement[] reps = new Replacement[strings.length];

        try {
            for (int i = 0; i < reps.length; i++) {
                reps[i] = new Replacement(strings[i][0], strings[i][1]);
            }
        } catch (ToolException e) {
            throw new ConfigException(e,
                                      localRes.getString("Unable_to_initialize"));
        }
        setReplacements(reps);
    }

    private void initAppReplacements(File deployRoot)
            throws ConfigException {
        Replacement[] newReps = new Replacement[0];

        newReps = ConfigTool.createReplacements(
           deployRoot.getParent(),
           deployRoot.getAbsolutePath(),
           getJavaPath());
        setReplacements(newReps);
    }

    private void initEnhydraReplacements() throws ConfigException {
        Replacement[] newReps = new Replacement[4];

        try {
            newReps[0] = new Replacement(AT_ENHYDRA_DIR_AT, getEnhydraRoot());
            newReps[1] = new Replacement(AT_JDKDIR_AT, getJavaPath());
            newReps[2] = new Replacement(AT_OUTPUT_AT, getEnhydraRoot());
            newReps[3] = new Replacement(AT_ENHYDRA_CLASSES_AT,
                                         getEnhydraRoot() + '/' + DIR_LIB
                                         + '/' + FILE_ENHYDRA_JAR);
            setReplacements(newReps);
        } catch (ToolException e) {
            throw new ConfigException(e,
                                      localRes.getString("Unable_to_initialize"));
        }
    }

    private Replacement[] getTransformedReplacements() {
        final String  AT = (new String()) + '@';
        final String  _PATH = "_PATH@";            // nores
        final String  OS_ = "@OS_";                // nores
        final String  SHELL_ = "@SHELL_";          // nores
        final String  JAVA_ = "@JAVA_";            // nores

        //
        Vector        transVector = new Vector();
        Replacement[] trans = new Replacement[0];
        String        cursorFind = new String();
        String        newFind = new String();
        String        path = new String();

        for (int i = 0; i < getReplacements().length; i++) {
            transVector.addElement(getReplacements()[i]);
            cursorFind = getReplacements()[i].getFind();
            if (cursorFind.startsWith(AT) && cursorFind.endsWith(_PATH)) {
                if (!cursorFind.startsWith(OS_)) {
                    path = getReplacements()[i].getReplaceWith()[0];
                    path = FileUtil.toCurrentPath(path);
                    try {
                        newFind = OS_ + cursorFind.substring(1);
                        Replacement osPath = new Replacement(newFind, path);

                        transVector.addElement(osPath);
                    } catch (ToolException e) {
                        echoWrite(e);
                    }
                }
                if (!cursorFind.startsWith(SHELL_)) {
                    newFind = SHELL_ + cursorFind.substring(1);
                    path = getReplacements()[i].getReplaceWith()[0];
                    path = FileUtil.toShellPath(path);
                    try {
                        Replacement shellPath = new Replacement(newFind,
                                                                path);

                        transVector.addElement(shellPath);
                    } catch (ToolException e) {
                        echoWrite(e);
                    }
                }
                if ((!cursorFind.startsWith(JAVA_)) || cursorFind.equals(JAVA_PATH)) {
                    newFind = JAVA_ + cursorFind.substring(1);
                    path = getReplacements()[i].getReplaceWith()[0];
                    path = FileUtil.toJavaPath(path);
                    try {
                        Replacement javaRep = new Replacement(newFind, path);
                        transVector.addElement(javaRep);
                    } catch (ToolException e) {
                        echoWrite(e);
                    }
                }
            }
        }
        trans = (Replacement[]) transVector.toArray(trans);
        transVector.removeAllElements();
        return trans;
    }

}
